/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef DEFORMATIONS_HH
#define DEFORMATIONS_HH

#include "MeshHistory.h"
#include "Slider.hh"
#include <gtkmm/table.h>
#include <libglademm.h>

namespace SharpConstruct
{
	namespace GInterface
	{
		class Deformations
		{
		public:
			Deformations();

			void Initialize( Glib::RefPtr< Gnome::Glade::Xml > );

			void Subdivide();
			void FlipNormals();
			void Unify();
			void ToTriangles();
		private:
			bool x_() const;
			bool y_() const;
			bool z_() const;
			bool preview_() const;

			void on_mirror_();
			
			void before_deformation_( const double );
			void revert_to_copy_();
			void after_deformation_( const double );

			void on_move_( const double );
			void on_scale_( const double );
			void on_deflate_( const double );
			void on_spherize_( const double );
			void on_smooth_( const double );
			void on_flood_( const double );

			Glib::RefPtr< Gnome::Glade::Xml > glade_;

			Slider move_;
			Slider scale_;
			Slider deflate_;
			Slider spherize_;
			Slider smooth_;
			Slider flood_;

			MeshHistory::PartialMesh mesh_copy_;
		};
	}
}

#endif
