/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Brush.h"
#include "Mesh.h"
#include "Optimized.h"
#include "PinchBrush.hh"

using namespace SharpConstruct;

PinchBrush::PinchBrush( SculptPhase, Mesh& mesh )
: mesh_( mesh )
{
	mesh_.ModifiedVertexIndices().clear();
}

void PinchBrush::operator()( SculptPhase, EditData e )
{
	mesh_.FindActiveVertices( active_data_, e );

	if( mesh_.CurrentBrush().PositionEdit() )
	{
		for( unsigned i = 0; i < active_data_.size(); i++ )
		{
			float str = active_data_[ i ].Strength() * -mesh_.CurrentBrush().Strength();
			mesh_.VertexLocations()[ active_data_[ i ].Index() ] +=
				( e.Center - mesh_.VertexLocations()[ active_data_[ i ].Index() ] ) * str;
		}
	}
}

void PinchBrush::Postwork( SculptPhase )
{
	if( mesh_.CurrentBrush().PositionEdit() )
	{
		mesh_.RecalculateDamaged();
		mesh_.RecalculateModifiedNormals();
	}
}
